# CheckSemantics.pm
#
# This file is a module part of VNUML package.
#
# Author: Fermn Galn Mrquez (galan@dit.upm.es)
# Copyright (C) 2005, 	DIT-UPM
# 			Departamento de Ingenieria de Sistemas Telematicos
#			Universidad Politecnica de Madrid
#			SPAIN
#			
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# An online copy of the licence can be found at http://www.gnu.org/copyleft/gpl.html

# CheckSemantincs implements the needed methods to check the VNUML XML specification before
# starting processing

package VNUML::CheckSemantics;
require(Exporter);

@ISA = qw(Exporter);
@EXPORT = qw(check_doc);

use strict;
use NetAddr::IP;
use VNUML::FileChecks;
use VNUML::IPChecks;
use VNUML::NetChecks;
use VNUML::TextManipulation;

# check_doc
#
# Arguments:
#
# - the DataHandler object reference that containts the document to be checked
# - the binaries_path hash reference
#
# Checks additional semantics in VNUML file that can not be
# checked during XML validation process.
# Currently this check consist in:
#
#   - to control source file version
#   - ensure that the ssh_version is valid
#   - valid network, mask, and offset for vm management
#   - exactly one <mgmt_net> child for the <vm_mgmt> tag if and only if
#     <vm_mgmt> tag has attribute type "net".
#   - The address of the mgmt_net is configured on an interface on the host machine
#   - there are no duplicated net names (<net>)
#   - external interfaces (external attribute of <net>) are actually physical interfaces
#     (they exist in the OS host enviroment)
#   - there is not duplicated UML names (<vm>)
#   - there is not duplicated interfaces in the same UML (<if>)
#   - no vm is connected to the vmmgmt_netname, used only for vm management
#   - there is not <if> with id 0 in any UML (reserved for management),
#     if not <mng_if>no</mng_if> or vmmgmt_type eq 'none'
#   - net attribute of <if> and <hostif> refers to existing <net>
#   - there is not duplicated <physicalif> names (we would be defining twice the same thing)
#     and they exists in the OS host enviroment
#   - <vm> and <net> names max of $max_name_length characters
#   - <ipv4> valid addresses
#   - <ipv6> valid addresses
#   - check IPv4 or IPv6 addresses for destination and gateway in <route> tag
#   - check ip attribute of <physicalif> for a valid IPv4 address 
#   - check IPv4 mask in <ipv4> and <physicalif> mask attributes
#   - <net type="ppp"> has exactly to virtual machine interfaces connected to it
#   - only <net type="ppp"> networks has <bw> tag
#   - <bw> is an integer number (bps)
#   - <xterm> only when <con0>xterm</con0> is being used
# 
# Check valid filename:
#
# - <ssh_key> value
# - <shell> value
# - <tun_device> value 
# - <filesystem> value
# - <kernel> value
# - <filetree> value (as directory) and "root" attribute (as directory)
# - <tun_device> value (if needed)
# - <basedir>
# 
# Check file/directory existence:
#
# - <ssh_key> value
# - <filesystem> value
# - <kernel> value
# - <filetree> value (as directory)
# - <tun_device> value (as character device) (if needed)
# - <basedir>
#
# Check rwx of files: 
# 
# - <kernel> value 
# - <tun_device> value (if needed)
#
# The functions returns error string describing problem or 1 if all is right
# 
# Fields used:
#
#    - default_filesystem
#    - default_filesystem_type
#    - default_kernel
#    - simname
#    - vmmgmt_type
#    - vmmgmt_net
#    - vmmgmt_mask
#    - vmmgmt_offset
#    - vmmgmt_netname
#    - shell
#    - host_mapping
#    - automac_offset
#    - ssh_version
#    - tundevice
#    - basedir
#
sub check_doc {
	
	my $max_name_length = 7;
	
	# Get arguments
	my $dh= shift;
	my $bp = shift;
	my $uid = shift;
	
	my $doc = $dh->get_doc;
	my @vm_ordered = $dh->get_vm_ordered;

	my $is_root = $> == 0 ? 1 : 0;
	my $uid_name = $is_root ? getpwuid($uid) : getpwuid($>);
      
    # 1. Version control
    my $version_list = $doc->getElementsByTagName("version");
       if ($version_list->getLength == 0) {
	   return "version of source file is missing";
    }
    my $version_tag = &text_tag($version_list->item(0));

    # Since version 1.2.1 only mayor version numbers (i.e, 1.2) are compared
    $dh->get_version =~ /^(\d\.\d+)/;
    my $version_mayor = $1;
    $version_tag =~ /^(\d\.\d+)/;
    my $version_tag_mayor = $1; 
    
    unless ($version_tag_mayor eq $version_mayor) {
	   return "mayor version numbers of source file ($version_tag) and parser (" . $dh->get_version . ") do not match";
    }

	# 2. Check ssh_version
    return $dh->get_ssh_version . " is not a valid ssh version"
		unless ($dh->get_ssh_version eq '1' || $dh->get_ssh_version eq '2');

    # 3. To check <ssh_key>
    my $ssh_key_list = $doc->getElementsByTagName("ssh_key");
    for (my $i = 0; $i < $ssh_key_list->getLength; $i++) {
	   my $ssh_key = &do_path_expansion(&text_tag($ssh_key_list->item($i)));
	   return "$ssh_key is not a valid absolute filename" unless &valid_absolute_filename($ssh_key);
	   return "$ssh_key (ssh key file) does not exist or is not readable" unless (-r $ssh_key);
    }

    # 4. To check <shell>
    #my $shell_list = $doc->getElementsByTagName("shell");
    #if ($shell_list->getLength != 0) {
	#   my $shell = &text_tag($shell_list->item(0));
	#   return "$shell is not a valid absolute filename" unless &valid_absolute_filename($shell);
    #}
    return $dh->get_shell . " (shell) is not a valid absolute filename" 
       unless &valid_absolute_filename($dh->get_shell);

    # 5. To check <tun_device>
    #my $tun_device_list = $doc->getElementsByTagName("tun_device");
    #if ($tun_device_list->getLength != 0) {
    #   my $tun_device = &text_tag($tun_device_list->item(0));
    #   return "$tun_device is not a valid absolute filename" unless &valid_absolute_filename($tun_device);
    #}
    if (&tundevice_needed($doc,$dh->get_vmmgmt_type,$dh->get_vm_ordered)) {
       return $dh->get_tun_device . " (tun_device) is not a valid absolute filename" 
          unless &valid_absolute_filename($dh->get_tun_device);
       return  $dh->get_tun_device . " (tun_device) does not exist or is not readable (user $uid_name)"
          unless (-r $dh->get_tun_device);
       return  $dh->get_tun_device . " (tun_device) is not writeable (user $uid_name)"
          unless (-w _);
       return $dh->get_tun_device . " (tun_device) is not a valid character device file" 
          unless (-c _);
    }

   # 6. To check <basedir>
   return $dh->get_basedir . " (basedir) is not a valid absolute directory name" 
      unless &valid_absolute_directoryname($dh->get_basedir);
   return $dh->get_basedir . " (basedir) does not exist or is not readable (user $uid_name)"
      unless (-d $dh->get_basedir);

	# 7. To check <vm_mgmt>
	if ($dh->get_vmmgmt_type ne 'none') {
		return "<vm_mgmt> network attribute \"".$dh->get_vmmgmt_net."\" is invalid"
			unless (&valid_ipv4($dh->get_vmmgmt_net));
		return "<vm_mgmt> mask attribute \"".$dh->get_vmmgmt_mask."\" is invalid (must be between 8 and 30)"
			unless ($dh->get_vmmgmt_mask =~ /^\d+$/ && $dh->get_vmmgmt_mask >= 8 && $dh->get_vmmgmt_mask <= 30);
		return "<vm_mgmt> offset attribute ".$dh->get_vmmgmt_offset." is too large for mask ".$dh->get_vmmgmt_mask
			if ($dh->get_vmmgmt_mask !~ /^\d+$/ || $dh->get_vmmgmt_offset > (1 << (32 - $dh->get_vmmgmt_mask)) - 3);
	}
	if ($dh->get_vmmgmt_type eq 'private') {
		return "<vm_mgmt> offset attribute must be a multiple of 4 for private management" if ($dh->get_vmmgmt_offset % 4 != 0);
	}
	my $vmmgmt_list = $doc->getElementsByTagName("vm_mgmt");
	my $vmmgmt_net_list;
	my $vmmgmt_net_list_len = 0;
	my $vmmgmt_hostmap_list;
	if ($vmmgmt_list->getLength == 1) {
		$vmmgmt_net_list = $vmmgmt_list->item(0)->getElementsByTagName("mgmt_net");
		$vmmgmt_net_list_len = $vmmgmt_net_list->getLength;
		$vmmgmt_hostmap_list = $vmmgmt_list->item(0)->getElementsByTagName("host_mapping");
	}
	if ($dh->get_vmmgmt_type eq 'net') {
		return "<vm_mgmt> element of type=\"net\" must have exactly one <mgmt_net> child element"
		if ($vmmgmt_net_list_len != 1);
		my $sock = &do_path_expansion($vmmgmt_net_list->item(0)->getAttribute("sock"));
		$> = $uid if ($is_root);
		return "$sock (sock) does not exist or is not readable (user $uid_name)" unless (-r $sock);
		return "$sock (sock) is not writeable (user $uid_name)" unless (-w _);
		return "$sock (sock) is not a valid socket" unless (-S _);
		$> = 0 if ($is_root);
		return "No interface on the host is configured for VM management with address " . $dh->get_vmmgmt_hostip . "/" . $dh->get_vmmgmt_mask
			unless (&hostip_exists($dh->get_vmmgmt_hostip, $dh->get_vmmgmt_mask));

	} else {
		return "<vm_mgmt> may only have a <mgmt_net> child if attribute type=\"net\"" if ($vmmgmt_net_list_len > 0);
	}
	return "<vm_mgmt> may not have a <host_mapping> child if attribute type=\"none\""
		if (defined($vmmgmt_hostmap_list) && $vmmgmt_hostmap_list->getLength > 0 && $dh->get_vmmgmt_type eq 'none');

   # 8. To check <net>
   # Hash for duplicated names detection
   my %net_names;
   
   # Hash for duplicated physical interface detection
   my %phyif_names;

   # To get list of defined <net>
   my $net_list = $doc->getElementsByTagName("net");

   # To process list
   for ( my $i = 0; $i < $net_list->getLength; $i++ ) {
      my $net = $net_list->item($i);

      # To get name attribute
      my $name = $net->getAttribute("name");

      # To check name length
      my $upper = $max_name_length + 1;
      return "net name $name is too long: max $max_name_length characters"
         if ($name =~ /^.{$upper,}$/);

      # To check if the same name has been seen before 
      if (defined($net_names{$name})) {
         return "duplicated net name: $name";
      }
	  elsif ($name =~ /_Mgmt$/) {
		return "the suffix \"_Mgmt\" has been designated for management networks and cannot be used in a <net> tag";
	  }
      else {
         $net_names{$name} = 1;
      }

      # To check external attribute (if present)
      # (this method is not very strong, only existence and duplication is 
      # checked; a best checking would be to analyze the full information 
      # returned by ifconfig command) 
      my $external_if = $net->getAttribute("external");
      unless ($external_if =~ /^$/) {
	    if (system($bp->{"ifconfig"} . " $external_if &> /dev/null")) {
	      return "in network $name, $external_if does not exist";
	    }
	    
	    # Check the VLAN attribute (to compose the physical name, for 
	    # duplication checking)
	    my $vlan = $net->getAttribute("vlan");
	    my $phy_name;
	    unless ($vlan =~ /^$/) {
	    	$phy_name = "$external_if.$vlan";
	    }
	    else {
	    	$phy_name = "$external_if";
	    }
	    
	    if (defined($phyif_names{$phy_name})) {
	    	return "two networks are attemping to use the same external physical interface: $phy_name";
	    }
	    else {
	    	$phyif_names{$phy_name} = 1;
	    }
      }
      
      # To check only two virtual machines for PPP nets
      my $type = $net->getAttribute("type");
      if ($type eq "ppp") {
         # Get all the ifs of the simulation
         my $machines = 0;
         my $if_list = $doc->getElementsByTagName("if");
         for ( my $j = 0; $j < $if_list->getLength; $j++ ) {
         	if ($if_list->item($j)->getAttribute("net") eq $name) {
         		$machines++;
         		last if ($machines > 3);
         	}
         }
         return "PPP $name net is connected to just one interface: PPP networks must be connected to exactly two interfaces" if ($machines < 2);
         return "PPP $name net is connected to more than two interface: PPP networks must be connected to exactly two interfaces"if ($machines > 2);         
      }
      
      if ($type ne "ppp") {
         # To check no-PPP networks doesn't have <bw> tag
         my $bw_list = $net->getElementsByTagName("bw");
         return "net $name is not a PPP network and only PPP networks can have a <bw> tag" if ($bw_list->getLength != 0)
      }
      else {
         # To check PPP networks has <bw> tag
         my $bw_list = $net->getElementsByTagName("bw");
         return "net $name is a PPP network and PPP networks must have a <bw> tag" if ($bw_list->getLength == 0)
      }

      my $sock = &do_path_expansion($net->getAttribute("sock"));
	  if ($sock !~ /^$/) {
		$> = $uid if ($is_root);
		return "$sock (sock) does not exist or is not readable (user $uid_name)" unless (-r $sock);
		return "$sock (sock) is not writeable (user $uid_name)" unless (-w _);
		return "$sock (sock) is not a valid socket" unless (-S _);
		$> = 0 if ($is_root);
	  }
      
   }
   
   # 9. To check <vm> and <if>
   
   # Hash for duplicated names detection
   my %vm_names;

   # Avoid duplicated hostfs directories
   my %hostfs_paths;
   
   for ( my $i = 0; $i < @vm_ordered; $i++) {
      my $vm = $vm_ordered[$i];

      # To get name attribute
      my $name = $vm->getAttribute("name");

      # To check name length
      my $upper = $max_name_length + 1;
      return "vm name $name is too long: max $max_name_length characters"
         if ($name =~ /^.{$upper,}$/);

      # To check if the same name has been seen before
      if (defined($vm_names{$name})) {
         return "duplicated vm name: $name";
      }
      else {
         $vm_names{$name} = 1;
      }

      # Hash for duplicated ids detection
      my %if_ids;

      # To get UML's interfaces list
      my $if_list = $vm->getElementsByTagName("if");

      # To process list
      for ( my $j = 0; $j < $if_list->getLength; $j++) {
         my $if = $if_list->item($j);

         # To get id attribute
         my $id = $if->getAttribute("id");

         # To check <mng_if>
	     my $mng_if_value = "";
	     my $mng_if_list = $vm->getElementsByTagName("mng_if");
	     if ($mng_if_list->getLength == 1) {
            $mng_if_value = &text_tag($mng_if_list->item(0));
		 }

	 # To check id 0 is not used
	 return "if id 0 in vm $name is not allowed while vm management is enabled unless <mng_if> is used"
	     if (($id == 0) && $dh->get_vmmgmt_type ne 'none' && ($mng_if_value ne "no"));

         # To check if the same id has been seen before
         if (defined($if_ids{$id})) {
             return "duplicated if id $id inside vm $name";
         }
         else {
            $if_ids{$id} = 1;
         }

         # To get net attribute
         my $net = $if->getAttribute("net");

	 # To check that there is a net with this name
	 unless ($net_names{$net} == 1) {
	    return "if id $id in vm $name net $net is not valid: it must be defined in a <net> tag";
	 }

      }

      #10. To check <filesystem> or <default_filesystem>                  
	  $> = $uid if ($is_root);
      my $filesystem_list = $vm->getElementsByTagName("filesystem");
      if ( $filesystem_list->getLength !=0 ) {
         my $filesystem = &do_path_expansion(&text_tag($filesystem_list->item(0)));
         my $filesystem_type = $filesystem_list->item(0)->getAttribute("type");
         if ($filesystem_type eq "hostfs") {
         	return "$filesystem (filesystem) is not a valid absolute directory name" unless &valid_absolute_directoryname($filesystem);
         	return "$filesystem (filesystem) is not does not exist or is not readable (user $uid_name)" unless (-d $filesystem);
         	return "$filesystem (filesystem) is not readable/executable (user $uid_name)" unless (-r _ && -x _);
            if ($hostfs_paths{&chompslash($filesystem)} == 1) {
               return "the same hostfs directory (" . &chompslash($filesystem) . ") is being used in more than one virtual machine";
            }
            else {
               $hostfs_paths{&chompslash($filesystem)} = 1;
            }
         	
         }
         else {
            return "$filesystem (filesystem) is not a valid absolute filename" unless &valid_absolute_filename($filesystem);
            return "$filesystem (filesystem) does not exist or is not readable (user $uid_name)" unless (-r $filesystem);
						if ($filesystem_type eq "direct") {
              return "$filesystem (filesystem) is not writeable (user $uid_name)" unless (-w $filesystem);
						}
         }
      }
      else {
         return "default filesystem " . $dh->get_default_filesystem . " is not a valid absolute filename" 
            unless &valid_absolute_filename($dh->get_default_filesystem);
         return "default filesystem " . $dh->get_default_filesystem . " does not exist or is not readable (user $uid_name)" 
            unless (-r $dh->get_default_filesystem);
      }

      #11. To check <kernel> or <default_kernel>
      my $kernel_list = $vm->getElementsByTagName("kernel");
      if ( $kernel_list->getLength != 0 ) {
		 my $kernel = $kernel_list->item(0);
         my $kernel_exe = &do_path_expansion(&text_tag($kernel));
         my $kernel_initrd = &do_path_expansion($kernel->getAttribute("initrd"));
         return "$kernel_exe (kernel) is not a valid absolute filename" unless &valid_absolute_filename($kernel_exe);
         return "$kernel_exe (kernel) does not exist or is not readable/executable (user $uid_name)" unless (-r $kernel_exe && -x _);
		 if ($kernel_initrd !~ /^$/) {
			 return "$kernel_initrd (initrd) is not a valid absolute filename" unless &valid_absolute_filename($kernel_initrd);
			 return "$kernel_initrd (initrd) does not exist or is not readable (user $uid_name)" unless (-r $kernel_initrd);
		 }
      }
      else  { 
         return "default kernel " . $dh->get_default_kernel . " is not a valid absolute filename" 
             unless &valid_absolute_filename($dh->get_default_kernel);
         return "default kernel " . $dh->get_default_kernel . " does not exist or is not readable/executable (user $uid_name)"
             unless (-r $dh->get_default_kernel && -x _);
		 if ($dh->get_default_initrd !~ /^$/) {
			 return "default initrd " . $dh->get_default_initrd . " is not a valid absolute filename" 
				 unless &valid_absolute_filename($dh->get_default_initrd);
			 return "default initrd " . $dh->get_default_initrd . " does not exist or is not readable (user $uid_name)"
				 unless (-r $dh->get_default_initrd);
		 }
      }
	  $> = 0 if ($is_root);

	  #12. Check users and groups
      my $user_list = $vm->getElementsByTagName("user");
	  for (my $j = 0; $j < $user_list->getLength; $j++) {
		 my $user = $user_list->item($j);
		 my $username = $user->getAttribute("username");
		 my $effective_group = $user->getAttribute("group");
		 return "Invalid username $username"
			 unless ($username =~ /[A-Za-z0-9_]+/);
		 my $group_list = $user->getElementsByTagName("group");
		 my %user_groups;
		 for (my $k = 0; $k < $group_list->getLength; $k++) {
			my $group = &text_tag($group_list->item($k));
			$user_groups{$group} = 1;
			return "Invalid group $group for user $username"
				unless ($group =~ /[A-Za-z0-9_]+/);
		 }
         return "Effective group " . $effective_group . " does not exist as a <group> tag for user $username"
             unless ($user_groups{$effective_group} eq '' || $user_groups{$effective_group});
	  }

      #13. To check <filetree>
      my $filetree_list = $vm->getElementsByTagName("filetree");

      # To process list
      for ( my $j = 0; $j < $filetree_list->getLength; $j++) {
         my $filetree = &text_tag($filetree_list->item($j));
         my $root = $filetree_list->item($j)->getAttribute("root");
         # Composed name
         my $filetree_effective = &do_path_expansion(&chompslash($dh->get_basedir) . "/" . &remove_heading_slash($filetree));
         # Checkings
         return "$filetree (filetree) is not a valid absolute directory name" 
            unless &valid_absolute_directoryname($filetree_effective);
         return "$filetree_effective (filetree) is not readable/executable (user $uid_name)"
            unless (-r $filetree_effective && -x _);
         return "$filetree (filetree) is not a valid directory in " . $dh->get_basedir
            unless (-d _);
         return "$root (root) is not a valid absolute directory name" unless &valid_absolute_directoryname($root);
      }

   }

   # 14. To check <hostif>
   my $hostif_list = $doc->getElementsByTagName("hostif");

   # To process list
   for ( my $i = 0; $i < $hostif_list->getLength; $i++) {
      my $hostif = $hostif_list->item($i);

      # To get net attribute
      my $net = $hostif->getAttribute("net");

      # To check that there is a net with this name
      unless ($net_names{$net} == 1) {
	    return "hostif net $net is not valid: it must be defined in a <net> tag";
      }

   }

   # 15. To check <physicalif>
   # Hash for duplicated names detection
   my %phyif_names_ipv4;
   my %phyif_names_ipv6;

   # To get list of defined <net>
   my $phyif_list = $doc->getElementsByTagName("physicalif");

   # To process list
   for ( my $i = 0; $i < $phyif_list->getLength; $i++ ) {
      my $phyif = $phyif_list->item($i);

      # To get name and type attribute
      my $name = $phyif->getAttribute("name");
      my $type = $phyif->getAttribute("type");

      if ($type eq "ipv6") {
         # IPv6 interface
         
         # To check if the same name has been seen before in IPv6 configurations
         if ($phyif_names_ipv6{$name} == 1) {
            return "duplicated physicalif name for IPv6 configuration: $name";
         }
         else {
            $phyif_names_ipv6{$name} = 1;
         }
         
         # It exists?
         if (system($bp->{"ifconfig"} . " $name &> /dev/null")) {
	        return "physicalif $name does not exists";
         }
         
         # To get ip attribute
         my $ip = $phyif->getAttribute("ip");
      
         # To check if valid IPv6 address
         unless (&valid_ipv6_with_mask($ip)) {
            return "'$ip' is not a valid IPv6 address with mask (/64 for example) in a <physicalif> ip";
         }
         
         # To get gw attribute
         my $gw = $phyif->getAttribute("gw");
         
         # To check if valid IPv6 address
         # Note the empty gw is allowable. This attribute is #IMPLIED in DTD and
         # the physicalif_config in vnumlparser.pl deals rightfully with emtpy values
         unless ($gw =~ /^$/ ) {
            unless (&valid_ipv6($gw)) {
               return "'$gw' is not a valid IPv6 address in a <physicalif> gw";
            }
         }         
      }
      else {
      	 # IPv4 interface
      	 
         # To check if the same name has been seen before in IPv4 configurations
         if ($phyif_names_ipv4{$name} == 1) {
            return "duplicated physicalif name for IPv4 configuration: $name";
         }
         else {
            $phyif_names_ipv4{$name} = 1;
         }

         # It exists?
         if (system($bp->{"ifconfig"} . " $name &> /dev/null")) {
	        return "physicalif $name does not exists";
         }

         # To get ip attribute
         my $ip = $phyif->getAttribute("ip");
      
         # To check if valid IPv4 address
         unless (&valid_ipv4($ip)) {
            return "'$ip' is not a valid IPv4 address in a <physicalif> ip";
         }

         # To get mask attribute
         my $mask = $phyif->getAttribute("mask");
         $mask="255.255.255.0" if ($mask =~ /^$/);

         # To check if valid IPv4 mask
         unless (&valid_mask($mask)) {
            return "'$mask' is not a valid IPv4 netmask in a <physicalif> mask attribute";
         }
         
         # To get gw attribute
         my $gw = $phyif->getAttribute("gw");
         
         # To check if valid IPv4 address
         # Note the empty gw is allowable. This attribute is #IMPLIED in DTD and
         # the physicalif_config in vnumlparser.pl deals rightfully with emtpy values
         unless ($gw =~ /^$/ ) {
            unless (&valid_ipv4($gw)) {
               return "'$gw' is not a valid IPv4 address in a <physicalif> gw";
            }
         }
      }
   }

   # 16. To check IPv4 addresses
   my $ipv4_list = $doc->getElementsByTagName("ipv4");
   for ( my $i = 0; $i < $ipv4_list->getLength; $i++ ) {
      my $ipv4 = &text_tag($ipv4_list->item($i));
      unless (&valid_ipv4($ipv4)) {
         return "'$ipv4' is not a valid IPv4 address (Z.Z.Z.Z) in a <ipv4>";
      }
      my $mask = $ipv4_list->item($i)->getAttribute("mask");
      unless (($mask eq '')||(&valid_mask($mask))) {
         return "'$mask' is not a valid IPv4 netmask in a <ipv4> mask attribute";
      }
   }

   # 17. To check IPv6 addresses
   my $ipv6_list = $doc->getElementsByTagName("ipv6");
   for ( my $i = 0; $i < $ipv6_list->getLength; $i++ ) {
      my $ipv6 = &text_tag($ipv6_list->item($i));
      unless (&valid_ipv6_with_mask($ipv6)) {
         return "'$ipv6' is not a valid IPv6 address (only Z:Z:Z:Z:Z:Z:Z:Z/M form is supported by the time) in a <ipv6>";
      }
   }

   # 18. To check addresses related with <route> tag
   my $route_list = $doc->getElementsByTagName("route");
   for ( my $i = 0; $i < $route_list->getLength; $i++ ) {
      my $route_dest = &text_tag($route_list->item($i));
      my $route_gw = $route_list->item($i)->getAttribute("gw");
      my $route_inet = $route_list->item($i)->getAttribute("type");
      if ($route_inet eq "inet") {
         unless (($route_dest eq "default")||(&valid_ipv4_with_mask($route_dest))) {
            return "'$route_dest' is not a valid IPv4 address with mask (Z.Z.Z.Z/M) in a <route>";
         }
         unless (&valid_ipv4($route_gw)) {
            return "'$route_gw' is not a valid IPv4 address (Z.Z.Z.Z) for a <route> gw";
         }
      }
      elsif ($route_inet eq "inet6") {
         unless (($route_dest eq "default")||(&valid_ipv6_with_mask($route_dest))) {
            return "'$route_dest' is not a valid IPv6 address with mask (only Z:Z:Z:Z:Z:Z:Z:Z/M is supported by the time) in a <route>";
         }
         unless (&valid_ipv6($route_gw)) {
            return "'$route_gw' is not a valid IPv6 address (only Z:Z:Z:Z:Z:Z:Z:Z is supported by the time) in a <route> gw";
         } 
      }
      else {
         return "$route_inet is not a valid <route> type";
      }

   }

   # 19. To check <bw>
   my $bw_list = $doc->getElementsByTagName("bw");
   for ( my $i = 0; $i < $bw_list->getLength; $i++ ) {
      my $bw = &text_tag($bw_list->item($i));
      return "<bw> value $bw is not a valid integer number" unless ($bw =~ /^\d+$/);
   }
   
   # 20. Check <xterm>
   my $boot_list = $doc->getElementsByTagName("boot");
   for ( my $i = 0; $i < $boot_list->getLength; $i++ ) {
      my $boot = $boot_list->item($i);

	  # Proccess <xterm> tag (if present). 
	  my $xterm_list = $boot->getElementsByTagName("xterm");
	  if ($xterm_list->getLength == 1) {

	     # Proccess <con0> tag 
	     my $con0_list = $boot->getElementsByTagName("con0");
	     if (($con0_list->getLength != 1) || (&text_tag($con0_list->item(0)) ne "xterm")) {
            return "you only can use <xterm> with <con0>xterm</con0>"
	     }
	     
	  }

   }

   return 0;

}

#
# hostip_exists
#
# Parameters:
#  - ip address
#  - mask
#
# returns true if an interface on the host machine
# is configured with the provided IP address and mask
#
sub hostip_exists {
	my $hostip = shift;
	my $hostmask = shift;
	my $ip = NetAddr::IP->new($hostip,$hostmask);

	my $found_address = 0;
	my $pipe_cmd = "ifconfig|";
	open (my $pipe, $pipe_cmd);
	while (<$pipe>) {
		if (/addr:(\d+\.\d+\.\d+\.\d+).+Mask:(\d+\.\d+\.\d+\.\d+)/) {
			my $ifip = NetAddr::IP->new($1,$2);
			if ($ifip == $ip) {
				$found_address = 1;
				last;
			}
		}
	}
	close($pipe);
	return $found_address;
}

1;
